from frictionless import describe, Detector, validate, extract, Pipeline, steps, Resource, transform
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from datetime import datetime as dt

# First we fix the csv file, because the second row contains the column information
# The fix is to swap the first and the second line of the csv file
with open("odala_crowdflowobserved_historic_data.csv", "r") as f:
    first_line = f.readline()
    second_line = f.readline()
    other_lines = f.read()
if second_line.startswith("id"):
    with open("odala_crowdflowobserved_historic_data.csv", "w") as f:
        f.writelines([second_line, first_line, other_lines])
        f.truncate()

# Descibe the data and detect missing values
detector = Detector(field_missing_values=[""])
resource = describe("odala_crowdflowobserved_historic_data.csv", detector=detector)
resource.to_yaml("odala_crowdflowobserved_historic_data.resource.yaml")

# Check the description of the data on correct types and columns
print("Description of the data table:")
print(describe("odala_crowdflowobserved_historic_data.resource.yaml"))

# Validate the data table

report = validate(Resource("odala_crowdflowobserved_historic_data.resource.yaml"))
print(report.to_summary())
if not report.valid:
    print(report.flatten(["rowNumber", "fieldNumber", "type"]))
else:
    print("CSV file validated using frictionless.")
    
    # Create a data table with all distinct names of sensors
    source = Resource("odala_crowdflowobserved_historic_data.resource.yaml")
    names = transform(
        source,
        steps=[
            steps.table_normalize(),
            steps.field_filter(names=["name"]),
            steps.row_subset(subset="distinct", field_name="name"),
        ]
    )

    # Create a data table with all dates, on which the First Sensor collected data
    source = Resource("odala_crowdflowobserved_historic_data.resource.yaml")
    dates = transform(
        source,
        steps=[
            steps.table_normalize(),
            steps.row_search(regex=extract(names)["odala_crowdflowobserved_historic_data"][0]["name"]),
            steps.field_filter(names=["TimeInstant"]),
            steps.cell_convert(field_name='TimeInstant', function=lambda v: dt.combine(v.date(), dt.min.time())),
            steps.row_subset(subset="distinct", field_name="TimeInstant"),
        ]
    )

    print("Names of the Sensors:")
    print(names.to_view())
    print("For the Plot we use the Sensor: ", extract(names)["odala_crowdflowobserved_historic_data"][0]["name"])

    # Create subplots with 4 rows and 4 columns
    fig, axs = plt.subplots(4,4, figsize=(15,21))

    axs = axs.flat

    i = 0
    # Extract the data collected in august 2023
    for date in extract(dates)["odala_crowdflowobserved_historic_data"]:
        if date["TimeInstant"].month == 8 and date["TimeInstant"].year == 2023:
            print("Analyzing data with date in August 2023: ", date["TimeInstant"].date())
            
            # Create a data table containing the peoplecount, timeinstant and location values of the first sensor on the specific day
            source = Resource("odala_crowdflowobserved_historic_data.resource.yaml")
            data = transform(
                source,
                steps=[
                    steps.table_normalize(),
                    steps.row_search(regex=extract(names)["odala_crowdflowobserved_historic_data"][0]["name"]),
                    steps.row_search(regex=str(date["TimeInstant"].date())),
                    steps.field_filter(names=["name", "peopleCountHighRange", "peopleCountLowerRange", "peopleCountMediumRange", "TimeInstant", "location"]),
                ]
            )

            # Get the data table as rows
            rows = extract(data)["odala_crowdflowobserved_historic_data"]

            # Create the plot
            axs[i].set_title("CrowdFlow on " + str(date["TimeInstant"].date()))
            axs[i].xaxis.set_major_formatter(mdates.DateFormatter('%H:%M'))
            axs[i].set_ylim((0,160))
            axs[i].hist([x["TimeInstant"] for x in rows], weights=[x["peopleCountHighRange"] for x in rows], bins=100)
            axs[i].hist([x["TimeInstant"] for x in rows], weights=[x["peopleCountMediumRange"] for x in rows], bins=100)
            axs[i].hist([x["TimeInstant"] for x in rows], weights=[x["peopleCountLowerRange"] for x in rows], bins=100)
            axs[i].grid()
            for label in axs[i].get_xticklabels():
                label.set_rotation(30)
                label.set_ha('right')
            if i == 0:
                axs[i].legend(["High Range", "Medium Range", "Low Range"])
            i+=1

    plt.savefig("output.png")

