import matplotlib.pyplot as plt
import geopandas as gpd
from frictionless import describe, Detector, validate, steps, Resource, transform
from pyproj import Transformer

# Method to get a column of a resource with all None values removed
def load_column(resource, column_name):
    data = transform(
        resource,
        steps=[
            steps.table_normalize(),
            steps.field_filter(names=[column_name]),
            steps.row_filter(formula="{} != None".format(column_name))
        ]
    )
    return data

# Descibe the data and detect missing values
detector = Detector(field_missing_values=[""])
resource = describe("polizeidienststellen.csv", detector=detector)

# Alter the auto generated schema to have the correct types and save it as yaml
resource.schema.set_field_type("website", "string")
resource.to_yaml("polizeidienststellen.resource.yaml")

# validate the data
report = validate('polizeidienststellen.resource.yaml')

if report.valid:
    print("CSV file validated using frictionless")

    # Get the longitude column of the data table and convert it to pandas data frame
    resource = Resource("polizeidienststellen.resource.yaml")
    longitude = load_column(resource, "longitude")
    longitude = longitude.to_pandas()

    # Get the latitude column of the data table and convert it to pandas data frame
    resource = Resource("polizeidienststellen.resource.yaml")
    latitude = load_column(resource, "latitude")
    latitude = latitude.to_pandas()

    # Load the Schleswig-Holstein map
    sh_df = gpd.read_file("./kreise.min.geojson")

    # transform the coordinates of the police stations to the format used in the geojson
    transformer = Transformer.from_crs(crs_from="4326", crs_to="25832", always_xy=True)
    longitude, latitude = transformer.transform(longitude, latitude, errcheck=True)

    # plot the map
    axes = sh_df.plot()

    # plot the coordinates of the police stations onto the SH map
    axes.scatter(longitude, latitude, c='orange', s=0.5)

    # Turn x and y axis off and save figure
    plt.axis('off')
    plt.savefig('output.png', dpi=200)

else:
    print(report.flatten(["rowNumber", "fieldNumber", "type"]))